# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.action_module_action import ActionModuleAction
from ...types.action_module_dependency_request import ActionModuleDependencyRequest
from ...types.action_module_list_item import ActionModuleListItem
from ...types.action_module_secret_request import ActionModuleSecretRequest
from ...types.create_action_module_response_content import CreateActionModuleResponseContent
from ...types.get_action_module_actions_response_content import GetActionModuleActionsResponseContent
from ...types.get_action_module_response_content import GetActionModuleResponseContent
from ...types.get_action_modules_response_content import GetActionModulesResponseContent
from ...types.rollback_action_module_response_content import RollbackActionModuleResponseContent
from ...types.update_action_module_response_content import UpdateActionModuleResponseContent
from .raw_client import AsyncRawModulesClient, RawModulesClient

if typing.TYPE_CHECKING:
    from .versions.client import AsyncVersionsClient, VersionsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ModulesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawModulesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._versions: typing.Optional[VersionsClient] = None

    @property
    def with_raw_response(self) -> RawModulesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawModulesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionModuleListItem, GetActionModulesResponseContent]:
        """
        Retrieve a paginated list of all Actions Modules with optional filtering and totals.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Paging is disabled if parameter not sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionModuleListItem, GetActionModulesResponseContent]
            The action modules were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.actions.modules.list(
            page=1,
            per_page=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(page=page, per_page=per_page, request_options=request_options)

    def create(
        self,
        *,
        name: str,
        code: str,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        publish: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateActionModuleResponseContent:
        """
        Create a new Actions Module for reusable code across actions.

        Parameters
        ----------
        name : str
            The name of the action module.

        code : str
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        api_version : typing.Optional[str]
            The API version of the module.

        publish : typing.Optional[bool]
            Whether to publish the module immediately after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateActionModuleResponseContent
            The action module was created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.create(
            name="name",
            code="code",
        )
        """
        _response = self._raw_client.create(
            name=name,
            code=code,
            secrets=secrets,
            dependencies=dependencies,
            api_version=api_version,
            publish=publish,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionModuleResponseContent:
        """
        Retrieve details of a specific Actions Module by its unique identifier.

        Parameters
        ----------
        id : str
            The ID of the action module to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionModuleResponseContent
            The action module was retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Permanently delete an Actions Module. This will fail if the module is still in use by any actions.

        Parameters
        ----------
        id : str
            The ID of the Actions Module to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        code: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateActionModuleResponseContent:
        """
        Update properties of an existing Actions Module, such as code, dependencies, or secrets.

        Parameters
        ----------
        id : str
            The ID of the action module to update.

        code : typing.Optional[str]
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateActionModuleResponseContent
            The action module was updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id, code=code, secrets=secrets, dependencies=dependencies, request_options=request_options
        )
        return _response.data

    def list_actions(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]:
        """
        Lists all actions that are using a specific Actions Module, showing which deployed action versions reference this Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]
            The actions were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.actions.modules.list_actions(
            id="id",
            page=1,
            per_page=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list_actions(id, page=page, per_page=per_page, request_options=request_options)

    def rollback(
        self, id: str, *, module_version_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RollbackActionModuleResponseContent:
        """
        Rolls back an Actions Module's draft to a previously created version. This action copies the code, dependencies, and secrets from the specified version into the current draft.

        Parameters
        ----------
        id : str
            The unique ID of the module to roll back.

        module_version_id : str
            The unique ID of the module version to roll back to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RollbackActionModuleResponseContent
            The rollback was successful.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.actions.modules.rollback(
            id="id",
            module_version_id="module_version_id",
        )
        """
        _response = self._raw_client.rollback(id, module_version_id=module_version_id, request_options=request_options)
        return _response.data

    @property
    def versions(self):
        if self._versions is None:
            from .versions.client import VersionsClient  # noqa: E402

            self._versions = VersionsClient(client_wrapper=self._client_wrapper)
        return self._versions


class AsyncModulesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawModulesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._versions: typing.Optional[AsyncVersionsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawModulesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawModulesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionModuleListItem, GetActionModulesResponseContent]:
        """
        Retrieve a paginated list of all Actions Modules with optional filtering and totals.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Paging is disabled if parameter not sent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionModuleListItem, GetActionModulesResponseContent]
            The action modules were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.actions.modules.list(
                page=1,
                per_page=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(page=page, per_page=per_page, request_options=request_options)

    async def create(
        self,
        *,
        name: str,
        code: str,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        api_version: typing.Optional[str] = OMIT,
        publish: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateActionModuleResponseContent:
        """
        Create a new Actions Module for reusable code across actions.

        Parameters
        ----------
        name : str
            The name of the action module.

        code : str
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        api_version : typing.Optional[str]
            The API version of the module.

        publish : typing.Optional[bool]
            Whether to publish the module immediately after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateActionModuleResponseContent
            The action module was created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.create(
                name="name",
                code="code",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            code=code,
            secrets=secrets,
            dependencies=dependencies,
            api_version=api_version,
            publish=publish,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetActionModuleResponseContent:
        """
        Retrieve details of a specific Actions Module by its unique identifier.

        Parameters
        ----------
        id : str
            The ID of the action module to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetActionModuleResponseContent
            The action module was retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Permanently delete an Actions Module. This will fail if the module is still in use by any actions.

        Parameters
        ----------
        id : str
            The ID of the Actions Module to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        code: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionModuleSecretRequest]] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionModuleDependencyRequest]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateActionModuleResponseContent:
        """
        Update properties of an existing Actions Module, such as code, dependencies, or secrets.

        Parameters
        ----------
        id : str
            The ID of the action module to update.

        code : typing.Optional[str]
            The source code of the action module.

        secrets : typing.Optional[typing.Sequence[ActionModuleSecretRequest]]
            The secrets to associate with the action module.

        dependencies : typing.Optional[typing.Sequence[ActionModuleDependencyRequest]]
            The npm dependencies of the action module.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateActionModuleResponseContent
            The action module was updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, code=code, secrets=secrets, dependencies=dependencies, request_options=request_options
        )
        return _response.data

    async def list_actions(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]:
        """
        Lists all actions that are using a specific Actions Module, showing which deployed action versions reference this Actions Module.

        Parameters
        ----------
        id : str
            The unique ID of the module.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ActionModuleAction, GetActionModuleActionsResponseContent]
            The actions were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.actions.modules.list_actions(
                id="id",
                page=1,
                per_page=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list_actions(id, page=page, per_page=per_page, request_options=request_options)

    async def rollback(
        self, id: str, *, module_version_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RollbackActionModuleResponseContent:
        """
        Rolls back an Actions Module's draft to a previously created version. This action copies the code, dependencies, and secrets from the specified version into the current draft.

        Parameters
        ----------
        id : str
            The unique ID of the module to roll back.

        module_version_id : str
            The unique ID of the module version to roll back to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RollbackActionModuleResponseContent
            The rollback was successful.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.actions.modules.rollback(
                id="id",
                module_version_id="module_version_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.rollback(
            id, module_version_id=module_version_id, request_options=request_options
        )
        return _response.data

    @property
    def versions(self):
        if self._versions is None:
            from .versions.client import AsyncVersionsClient  # noqa: E402

            self._versions = AsyncVersionsClient(client_wrapper=self._client_wrapper)
        return self._versions



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FlowActionStripeAddress(UniversalBaseModel):
    line_1: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="line1"), pydantic.Field(alias="line1")
    ] = None
    line_2: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="line2"), pydantic.Field(alias="line2")
    ] = None
    postal_code: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="postalCode"), pydantic.Field(alias="postalCode")
    ] = None
    city: typing.Optional[str] = None
    state: typing.Optional[str] = None
    country: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count

from auth0.management import SetUserAuthenticationMethods


def test_users_authenticationMethods_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "users.authentication_methods.list_.0"
    client = get_client(test_id)
    client.users.authentication_methods.list(
        id="id",
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id, "GET", "/users/id/authentication-methods", {"page": "1", "per_page": "1", "include_totals": "true"}, 1
    )


def test_users_authenticationMethods_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "users.authentication_methods.create.0"
    client = get_client(test_id)
    client.users.authentication_methods.create(
        id="id",
        type="phone",
    )
    verify_request_count(test_id, "POST", "/users/id/authentication-methods", None, 1)


def test_users_authenticationMethods_set_() -> None:
    """Test set endpoint with WireMock"""
    test_id = "users.authentication_methods.set_.0"
    client = get_client(test_id)
    client.users.authentication_methods.set(
        id="id",
        request=[
            SetUserAuthenticationMethods(
                type="phone",
            )
        ],
    )
    verify_request_count(test_id, "PUT", "/users/id/authentication-methods", None, 1)


def test_users_authenticationMethods_delete_all() -> None:
    """Test deleteAll endpoint with WireMock"""
    test_id = "users.authentication_methods.delete_all.0"
    client = get_client(test_id)
    client.users.authentication_methods.delete_all(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/users/id/authentication-methods", None, 1)


def test_users_authenticationMethods_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "users.authentication_methods.get.0"
    client = get_client(test_id)
    client.users.authentication_methods.get(
        id="id",
        authentication_method_id="authentication_method_id",
    )
    verify_request_count(test_id, "GET", "/users/id/authentication-methods/authentication_method_id", None, 1)


def test_users_authenticationMethods_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "users.authentication_methods.delete.0"
    client = get_client(test_id)
    client.users.authentication_methods.delete(
        id="id",
        authentication_method_id="authentication_method_id",
    )
    verify_request_count(test_id, "DELETE", "/users/id/authentication-methods/authentication_method_id", None, 1)


def test_users_authenticationMethods_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "users.authentication_methods.update.0"
    client = get_client(test_id)
    client.users.authentication_methods.update(
        id="id",
        authentication_method_id="authentication_method_id",
    )
    verify_request_count(test_id, "PATCH", "/users/id/authentication-methods/authentication_method_id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.branding_theme_borders import BrandingThemeBorders
from ...types.branding_theme_colors import BrandingThemeColors
from ...types.branding_theme_fonts import BrandingThemeFonts
from ...types.branding_theme_identifiers import BrandingThemeIdentifiers
from ...types.branding_theme_page_background import BrandingThemePageBackground
from ...types.branding_theme_widget import BrandingThemeWidget
from ...types.create_branding_theme_response_content import CreateBrandingThemeResponseContent
from ...types.get_branding_default_theme_response_content import GetBrandingDefaultThemeResponseContent
from ...types.get_branding_theme_response_content import GetBrandingThemeResponseContent
from ...types.update_branding_theme_response_content import UpdateBrandingThemeResponseContent
from .raw_client import AsyncRawThemesClient, RawThemesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ThemesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawThemesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawThemesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawThemesClient
        """
        return self._raw_client

    def create(
        self,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBrandingThemeResponseContent:
        """
        Create branding theme.

        Parameters
        ----------
        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBrandingThemeResponseContent
            Branding settings successfully updated.

        Examples
        --------
        from auth0 import (
            Auth0,
            BrandingThemeBorders,
            BrandingThemeColors,
            BrandingThemeFontBodyText,
            BrandingThemeFontButtonsText,
            BrandingThemeFontInputLabels,
            BrandingThemeFontLinks,
            BrandingThemeFonts,
            BrandingThemeFontSubtitle,
            BrandingThemeFontTitle,
            BrandingThemePageBackground,
            BrandingThemeWidget,
        )

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.themes.create(
            borders=BrandingThemeBorders(
                button_border_radius=1.1,
                button_border_weight=1.1,
                buttons_style="pill",
                input_border_radius=1.1,
                input_border_weight=1.1,
                inputs_style="pill",
                show_widget_shadow=True,
                widget_border_weight=1.1,
                widget_corner_radius=1.1,
            ),
            colors=BrandingThemeColors(
                body_text="body_text",
                error="error",
                header="header",
                icons="icons",
                input_background="input_background",
                input_border="input_border",
                input_filled_text="input_filled_text",
                input_labels_placeholders="input_labels_placeholders",
                links_focused_components="links_focused_components",
                primary_button="primary_button",
                primary_button_label="primary_button_label",
                secondary_button_border="secondary_button_border",
                secondary_button_label="secondary_button_label",
                success="success",
                widget_background="widget_background",
                widget_border="widget_border",
            ),
            fonts=BrandingThemeFonts(
                body_text=BrandingThemeFontBodyText(
                    bold=True,
                    size=1.1,
                ),
                buttons_text=BrandingThemeFontButtonsText(
                    bold=True,
                    size=1.1,
                ),
                font_url="font_url",
                input_labels=BrandingThemeFontInputLabels(
                    bold=True,
                    size=1.1,
                ),
                links=BrandingThemeFontLinks(
                    bold=True,
                    size=1.1,
                ),
                links_style="normal",
                reference_text_size=1.1,
                subtitle=BrandingThemeFontSubtitle(
                    bold=True,
                    size=1.1,
                ),
                title=BrandingThemeFontTitle(
                    bold=True,
                    size=1.1,
                ),
            ),
            page_background=BrandingThemePageBackground(
                background_color="background_color",
                background_image_url="background_image_url",
                page_layout="center",
            ),
            widget=BrandingThemeWidget(
                header_text_alignment="center",
                logo_height=1.1,
                logo_position="center",
                logo_url="logo_url",
                social_buttons_layout="bottom",
            ),
        )
        """
        _response = self._raw_client.create(
            borders=borders,
            colors=colors,
            fonts=fonts,
            page_background=page_background,
            widget=widget,
            display_name=display_name,
            identifiers=identifiers,
            request_options=request_options,
        )
        return _response.data

    def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBrandingDefaultThemeResponseContent:
        """
        Retrieve default branding theme.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBrandingDefaultThemeResponseContent
            Branding theme successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.themes.get_default()
        """
        _response = self._raw_client.get_default(request_options=request_options)
        return _response.data

    def get(
        self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBrandingThemeResponseContent:
        """
        Retrieve branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBrandingThemeResponseContent
            Branding theme successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.themes.get(
            theme_id="themeId",
        )
        """
        _response = self._raw_client.get(theme_id, request_options=request_options)
        return _response.data

    def delete(self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.themes.delete(
            theme_id="themeId",
        )
        """
        _response = self._raw_client.delete(theme_id, request_options=request_options)
        return _response.data

    def update(
        self,
        theme_id: str,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBrandingThemeResponseContent:
        """
        Update branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBrandingThemeResponseContent
            Branding settings successfully updated.

        Examples
        --------
        from auth0 import (
            Auth0,
            BrandingThemeBorders,
            BrandingThemeColors,
            BrandingThemeFontBodyText,
            BrandingThemeFontButtonsText,
            BrandingThemeFontInputLabels,
            BrandingThemeFontLinks,
            BrandingThemeFonts,
            BrandingThemeFontSubtitle,
            BrandingThemeFontTitle,
            BrandingThemePageBackground,
            BrandingThemeWidget,
        )

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.branding.themes.update(
            theme_id="themeId",
            borders=BrandingThemeBorders(
                button_border_radius=1.1,
                button_border_weight=1.1,
                buttons_style="pill",
                input_border_radius=1.1,
                input_border_weight=1.1,
                inputs_style="pill",
                show_widget_shadow=True,
                widget_border_weight=1.1,
                widget_corner_radius=1.1,
            ),
            colors=BrandingThemeColors(
                body_text="body_text",
                error="error",
                header="header",
                icons="icons",
                input_background="input_background",
                input_border="input_border",
                input_filled_text="input_filled_text",
                input_labels_placeholders="input_labels_placeholders",
                links_focused_components="links_focused_components",
                primary_button="primary_button",
                primary_button_label="primary_button_label",
                secondary_button_border="secondary_button_border",
                secondary_button_label="secondary_button_label",
                success="success",
                widget_background="widget_background",
                widget_border="widget_border",
            ),
            fonts=BrandingThemeFonts(
                body_text=BrandingThemeFontBodyText(
                    bold=True,
                    size=1.1,
                ),
                buttons_text=BrandingThemeFontButtonsText(
                    bold=True,
                    size=1.1,
                ),
                font_url="font_url",
                input_labels=BrandingThemeFontInputLabels(
                    bold=True,
                    size=1.1,
                ),
                links=BrandingThemeFontLinks(
                    bold=True,
                    size=1.1,
                ),
                links_style="normal",
                reference_text_size=1.1,
                subtitle=BrandingThemeFontSubtitle(
                    bold=True,
                    size=1.1,
                ),
                title=BrandingThemeFontTitle(
                    bold=True,
                    size=1.1,
                ),
            ),
            page_background=BrandingThemePageBackground(
                background_color="background_color",
                background_image_url="background_image_url",
                page_layout="center",
            ),
            widget=BrandingThemeWidget(
                header_text_alignment="center",
                logo_height=1.1,
                logo_position="center",
                logo_url="logo_url",
                social_buttons_layout="bottom",
            ),
        )
        """
        _response = self._raw_client.update(
            theme_id,
            borders=borders,
            colors=colors,
            fonts=fonts,
            page_background=page_background,
            widget=widget,
            display_name=display_name,
            identifiers=identifiers,
            request_options=request_options,
        )
        return _response.data


class AsyncThemesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawThemesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawThemesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawThemesClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateBrandingThemeResponseContent:
        """
        Create branding theme.

        Parameters
        ----------
        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateBrandingThemeResponseContent
            Branding settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import (
            AsyncAuth0,
            BrandingThemeBorders,
            BrandingThemeColors,
            BrandingThemeFontBodyText,
            BrandingThemeFontButtonsText,
            BrandingThemeFontInputLabels,
            BrandingThemeFontLinks,
            BrandingThemeFonts,
            BrandingThemeFontSubtitle,
            BrandingThemeFontTitle,
            BrandingThemePageBackground,
            BrandingThemeWidget,
        )

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.themes.create(
                borders=BrandingThemeBorders(
                    button_border_radius=1.1,
                    button_border_weight=1.1,
                    buttons_style="pill",
                    input_border_radius=1.1,
                    input_border_weight=1.1,
                    inputs_style="pill",
                    show_widget_shadow=True,
                    widget_border_weight=1.1,
                    widget_corner_radius=1.1,
                ),
                colors=BrandingThemeColors(
                    body_text="body_text",
                    error="error",
                    header="header",
                    icons="icons",
                    input_background="input_background",
                    input_border="input_border",
                    input_filled_text="input_filled_text",
                    input_labels_placeholders="input_labels_placeholders",
                    links_focused_components="links_focused_components",
                    primary_button="primary_button",
                    primary_button_label="primary_button_label",
                    secondary_button_border="secondary_button_border",
                    secondary_button_label="secondary_button_label",
                    success="success",
                    widget_background="widget_background",
                    widget_border="widget_border",
                ),
                fonts=BrandingThemeFonts(
                    body_text=BrandingThemeFontBodyText(
                        bold=True,
                        size=1.1,
                    ),
                    buttons_text=BrandingThemeFontButtonsText(
                        bold=True,
                        size=1.1,
                    ),
                    font_url="font_url",
                    input_labels=BrandingThemeFontInputLabels(
                        bold=True,
                        size=1.1,
                    ),
                    links=BrandingThemeFontLinks(
                        bold=True,
                        size=1.1,
                    ),
                    links_style="normal",
                    reference_text_size=1.1,
                    subtitle=BrandingThemeFontSubtitle(
                        bold=True,
                        size=1.1,
                    ),
                    title=BrandingThemeFontTitle(
                        bold=True,
                        size=1.1,
                    ),
                ),
                page_background=BrandingThemePageBackground(
                    background_color="background_color",
                    background_image_url="background_image_url",
                    page_layout="center",
                ),
                widget=BrandingThemeWidget(
                    header_text_alignment="center",
                    logo_height=1.1,
                    logo_position="center",
                    logo_url="logo_url",
                    social_buttons_layout="bottom",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            borders=borders,
            colors=colors,
            fonts=fonts,
            page_background=page_background,
            widget=widget,
            display_name=display_name,
            identifiers=identifiers,
            request_options=request_options,
        )
        return _response.data

    async def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBrandingDefaultThemeResponseContent:
        """
        Retrieve default branding theme.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBrandingDefaultThemeResponseContent
            Branding theme successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.themes.get_default()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_default(request_options=request_options)
        return _response.data

    async def get(
        self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBrandingThemeResponseContent:
        """
        Retrieve branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBrandingThemeResponseContent
            Branding theme successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.themes.get(
                theme_id="themeId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(theme_id, request_options=request_options)
        return _response.data

    async def delete(self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.themes.delete(
                theme_id="themeId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(theme_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        theme_id: str,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBrandingThemeResponseContent:
        """
        Update branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBrandingThemeResponseContent
            Branding settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import (
            AsyncAuth0,
            BrandingThemeBorders,
            BrandingThemeColors,
            BrandingThemeFontBodyText,
            BrandingThemeFontButtonsText,
            BrandingThemeFontInputLabels,
            BrandingThemeFontLinks,
            BrandingThemeFonts,
            BrandingThemeFontSubtitle,
            BrandingThemeFontTitle,
            BrandingThemePageBackground,
            BrandingThemeWidget,
        )

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.branding.themes.update(
                theme_id="themeId",
                borders=BrandingThemeBorders(
                    button_border_radius=1.1,
                    button_border_weight=1.1,
                    buttons_style="pill",
                    input_border_radius=1.1,
                    input_border_weight=1.1,
                    inputs_style="pill",
                    show_widget_shadow=True,
                    widget_border_weight=1.1,
                    widget_corner_radius=1.1,
                ),
                colors=BrandingThemeColors(
                    body_text="body_text",
                    error="error",
                    header="header",
                    icons="icons",
                    input_background="input_background",
                    input_border="input_border",
                    input_filled_text="input_filled_text",
                    input_labels_placeholders="input_labels_placeholders",
                    links_focused_components="links_focused_components",
                    primary_button="primary_button",
                    primary_button_label="primary_button_label",
                    secondary_button_border="secondary_button_border",
                    secondary_button_label="secondary_button_label",
                    success="success",
                    widget_background="widget_background",
                    widget_border="widget_border",
                ),
                fonts=BrandingThemeFonts(
                    body_text=BrandingThemeFontBodyText(
                        bold=True,
                        size=1.1,
                    ),
                    buttons_text=BrandingThemeFontButtonsText(
                        bold=True,
                        size=1.1,
                    ),
                    font_url="font_url",
                    input_labels=BrandingThemeFontInputLabels(
                        bold=True,
                        size=1.1,
                    ),
                    links=BrandingThemeFontLinks(
                        bold=True,
                        size=1.1,
                    ),
                    links_style="normal",
                    reference_text_size=1.1,
                    subtitle=BrandingThemeFontSubtitle(
                        bold=True,
                        size=1.1,
                    ),
                    title=BrandingThemeFontTitle(
                        bold=True,
                        size=1.1,
                    ),
                ),
                page_background=BrandingThemePageBackground(
                    background_color="background_color",
                    background_image_url="background_image_url",
                    page_layout="center",
                ),
                widget=BrandingThemeWidget(
                    header_text_alignment="center",
                    logo_height=1.1,
                    logo_position="center",
                    logo_url="logo_url",
                    social_buttons_layout="bottom",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            theme_id,
            borders=borders,
            colors=colors,
            fonts=fonts,
            page_background=page_background,
            widget=widget,
            display_name=display_name,
            identifiers=identifiers,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

CustomSigningKeyAlgorithmEnum = typing.Union[
    typing.Literal["RS256", "RS384", "RS512", "ES256", "ES384", "ES512", "PS256", "PS384", "PS512"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

ConnectionRequestTemplateSaml = str
"""
Custom XML template for SAML authentication requests. Supports variable substitution using @@variableName@@ syntax. When not provided, uses default SAML AuthnRequest template. See https://auth0.com/docs/authenticate/protocols/saml/saml-sso-integrations/configure-auth0-saml-service-provider#customize-the-request-template
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.get_guardian_factor_duo_settings_response_content import GetGuardianFactorDuoSettingsResponseContent
from .....types.set_guardian_factor_duo_settings_response_content import SetGuardianFactorDuoSettingsResponseContent
from .....types.update_guardian_factor_duo_settings_response_content import (
    UpdateGuardianFactorDuoSettingsResponseContent,
)
from .raw_client import AsyncRawSettingsClient, RawSettingsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSettingsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSettingsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSettingsClient
        """
        return self._raw_client

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGuardianFactorDuoSettingsResponseContent:
        """
        Retrieves the DUO account and factor configuration.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGuardianFactorDuoSettingsResponseContent
            DUO settings successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.factors.duo.settings.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def set(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetGuardianFactorDuoSettingsResponseContent:
        """
        Set the DUO account configuration and other properties specific to this factor.

        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetGuardianFactorDuoSettingsResponseContent
            DUO settings successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.factors.duo.settings.set()
        """
        _response = self._raw_client.set(ikey=ikey, skey=skey, host=host, request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateGuardianFactorDuoSettingsResponseContent:
        """
        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateGuardianFactorDuoSettingsResponseContent
            DUO settings successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.factors.duo.settings.update()
        """
        _response = self._raw_client.update(ikey=ikey, skey=skey, host=host, request_options=request_options)
        return _response.data


class AsyncSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSettingsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSettingsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSettingsClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGuardianFactorDuoSettingsResponseContent:
        """
        Retrieves the DUO account and factor configuration.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGuardianFactorDuoSettingsResponseContent
            DUO settings successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.factors.duo.settings.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def set(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetGuardianFactorDuoSettingsResponseContent:
        """
        Set the DUO account configuration and other properties specific to this factor.

        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetGuardianFactorDuoSettingsResponseContent
            DUO settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.factors.duo.settings.set()


        asyncio.run(main())
        """
        _response = await self._raw_client.set(ikey=ikey, skey=skey, host=host, request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateGuardianFactorDuoSettingsResponseContent:
        """
        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateGuardianFactorDuoSettingsResponseContent
            DUO settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.factors.duo.settings.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(ikey=ikey, skey=skey, host=host, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .flow_action_salesforce_update_lead_action import FlowActionSalesforceUpdateLeadAction
from .flow_action_salesforce_update_lead_params import FlowActionSalesforceUpdateLeadParams
from .flow_action_salesforce_update_lead_type import FlowActionSalesforceUpdateLeadType


class FlowActionSalesforceUpdateLead(UniversalBaseModel):
    id: str
    alias: typing.Optional[str] = None
    type: FlowActionSalesforceUpdateLeadType
    action: FlowActionSalesforceUpdateLeadAction
    allow_failure: typing.Optional[bool] = None
    mask_output: typing.Optional[bool] = None
    params: FlowActionSalesforceUpdateLeadParams

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .directory_provisioning_mapping_item import DirectoryProvisioningMappingItem
from .synchronize_groups_enum import SynchronizeGroupsEnum


class UpdateDirectoryProvisioningResponseContent(UniversalBaseModel):
    connection_id: str = pydantic.Field()
    """
    The connection's identifier
    """

    connection_name: str = pydantic.Field()
    """
    The connection's name
    """

    strategy: str = pydantic.Field()
    """
    The connection's strategy
    """

    mapping: typing.List[DirectoryProvisioningMappingItem] = pydantic.Field()
    """
    The mapping between Auth0 and IDP user attributes
    """

    synchronize_automatically: bool = pydantic.Field()
    """
    Whether periodic automatic synchronization is enabled
    """

    synchronize_groups: typing.Optional[SynchronizeGroupsEnum] = None
    created_at: dt.datetime = pydantic.Field()
    """
    The timestamp at which the directory provisioning configuration was created
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    The timestamp at which the directory provisioning configuration was last updated
    """

    last_synchronization_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp at which the connection was last synchronized
    """

    last_synchronization_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of the last synchronization
    """

    last_synchronization_error: typing.Optional[str] = pydantic.Field(default=None)
    """
    The error message of the last synchronization, if any
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionClaimsSupported = typing.List[str]
"""
JSON array containing a list of the Claim Names of the Claims that the OpenID Provider MAY be able to supply values for. Note that for privacy or other reasons, this might not be an exhaustive list.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .flow_action_pipedrive_add_deal_params_fields import FlowActionPipedriveAddDealParamsFields
from .flow_action_pipedrive_add_deal_params_organization_id import FlowActionPipedriveAddDealParamsOrganizationId
from .flow_action_pipedrive_add_deal_params_person_id import FlowActionPipedriveAddDealParamsPersonId
from .flow_action_pipedrive_add_deal_params_stage_id import FlowActionPipedriveAddDealParamsStageId
from .flow_action_pipedrive_add_deal_params_user_id import FlowActionPipedriveAddDealParamsUserId


class FlowActionPipedriveAddDealParams(UniversalBaseModel):
    connection_id: str
    title: str
    value: typing.Optional[str] = None
    user_id: typing.Optional[FlowActionPipedriveAddDealParamsUserId] = None
    person_id: typing.Optional[FlowActionPipedriveAddDealParamsPersonId] = None
    organization_id: typing.Optional[FlowActionPipedriveAddDealParamsOrganizationId] = None
    stage_id: typing.Optional[FlowActionPipedriveAddDealParamsStageId] = None
    fields: typing.Optional[FlowActionPipedriveAddDealParamsFields] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ... import core
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_import_users_response_content import CreateImportUsersResponseContent
from .raw_client import AsyncRawUsersImportsClient, RawUsersImportsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersImportsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersImportsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersImportsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersImportsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        users: core.File,
        connection_id: str,
        upsert: typing.Optional[bool] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        send_completion_email: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateImportUsersResponseContent:
        """
        Import users from a <a href="https://auth0.com/docs/users/references/bulk-import-database-schema-examples">formatted file</a> into a connection via a long-running job. When importing users, with or without upsert, the `email_verified` is set to `false` when the email address is added or updated. Users must verify their email address. To avoid this behavior, set `email_verified` to `true` in the imported data.

        Parameters
        ----------
        users : core.File
            See core.File for more documentation

        connection_id : str
            connection_id of the connection to which users will be imported.

        upsert : typing.Optional[bool]
            Whether to update users if they already exist (true) or to ignore them (false).

        external_id : typing.Optional[str]
            Customer-defined ID.

        send_completion_email : typing.Optional[bool]
            Whether to send a completion email to all tenant owners when the job is finished (true) or not (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateImportUsersResponseContent
            Job successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.jobs.users_imports.create(
            connection_id="connection_id",
        )
        """
        _response = self._raw_client.create(
            users=users,
            connection_id=connection_id,
            upsert=upsert,
            external_id=external_id,
            send_completion_email=send_completion_email,
            request_options=request_options,
        )
        return _response.data


class AsyncUsersImportsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersImportsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersImportsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersImportsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        users: core.File,
        connection_id: str,
        upsert: typing.Optional[bool] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        send_completion_email: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateImportUsersResponseContent:
        """
        Import users from a <a href="https://auth0.com/docs/users/references/bulk-import-database-schema-examples">formatted file</a> into a connection via a long-running job. When importing users, with or without upsert, the `email_verified` is set to `false` when the email address is added or updated. Users must verify their email address. To avoid this behavior, set `email_verified` to `true` in the imported data.

        Parameters
        ----------
        users : core.File
            See core.File for more documentation

        connection_id : str
            connection_id of the connection to which users will be imported.

        upsert : typing.Optional[bool]
            Whether to update users if they already exist (true) or to ignore them (false).

        external_id : typing.Optional[str]
            Customer-defined ID.

        send_completion_email : typing.Optional[bool]
            Whether to send a completion email to all tenant owners when the job is finished (true) or not (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateImportUsersResponseContent
            Job successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.users_imports.create(
                connection_id="connection_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            users=users,
            connection_id=connection_id,
            upsert=upsert,
            external_id=external_id,
            send_completion_email=send_completion_email,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.list_user_blocks_by_identifier_response_content import ListUserBlocksByIdentifierResponseContent
from ..types.list_user_blocks_response_content import ListUserBlocksResponseContent
from .raw_client import AsyncRawUserBlocksClient, RawUserBlocksClient


class UserBlocksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUserBlocksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUserBlocksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUserBlocksClient
        """
        return self._raw_client

    def list_by_identifier(
        self,
        *,
        identifier: str,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUserBlocksByIdentifierResponseContent:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for a user with the given identifier (username, phone number, or email).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUserBlocksByIdentifierResponseContent
            User successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.user_blocks.list_by_identifier(
            identifier="identifier",
            consider_brute_force_enablement=True,
        )
        """
        _response = self._raw_client.list_by_identifier(
            identifier=identifier,
            consider_brute_force_enablement=consider_brute_force_enablement,
            request_options=request_options,
        )
        return _response.data

    def delete_by_identifier(self, *, identifier: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given identifier (username, phone number, or email).

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.user_blocks.delete_by_identifier(
            identifier="identifier",
        )
        """
        _response = self._raw_client.delete_by_identifier(identifier=identifier, request_options=request_options)
        return _response.data

    def list(
        self,
        id: str,
        *,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUserBlocksResponseContent:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Parameters
        ----------
        id : str
            user_id of the user blocks to retrieve.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUserBlocksResponseContent
            User block successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.user_blocks.list(
            id="id",
            consider_brute_force_enablement=True,
        )
        """
        _response = self._raw_client.list(
            id, consider_brute_force_enablement=consider_brute_force_enablement, request_options=request_options
        )
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        id : str
            The user_id of the user to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.user_blocks.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncUserBlocksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUserBlocksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUserBlocksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUserBlocksClient
        """
        return self._raw_client

    async def list_by_identifier(
        self,
        *,
        identifier: str,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUserBlocksByIdentifierResponseContent:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for a user with the given identifier (username, phone number, or email).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUserBlocksByIdentifierResponseContent
            User successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_blocks.list_by_identifier(
                identifier="identifier",
                consider_brute_force_enablement=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_by_identifier(
            identifier=identifier,
            consider_brute_force_enablement=consider_brute_force_enablement,
            request_options=request_options,
        )
        return _response.data

    async def delete_by_identifier(
        self, *, identifier: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given identifier (username, phone number, or email).

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_blocks.delete_by_identifier(
                identifier="identifier",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_by_identifier(identifier=identifier, request_options=request_options)
        return _response.data

    async def list(
        self,
        id: str,
        *,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUserBlocksResponseContent:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Parameters
        ----------
        id : str
            user_id of the user blocks to retrieve.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUserBlocksResponseContent
            User block successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_blocks.list(
                id="id",
                consider_brute_force_enablement=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            id, consider_brute_force_enablement=consider_brute_force_enablement, request_options=request_options
        )
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        id : str
            The user_id of the user to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_blocks.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .identity_provider_enum import IdentityProviderEnum


class Identity(UniversalBaseModel):
    """
    This must be provided to verify primary social, enterprise and passwordless email identities. Also, is needed to verify secondary identities.
    """

    user_id: str = pydantic.Field()
    """
    user_id of the identity to be verified.
    """

    provider: IdentityProviderEnum
    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    connection_id of the identity.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .integration_required_param_option import IntegrationRequiredParamOption
from .integration_required_param_type_enum import IntegrationRequiredParamTypeEnum


class IntegrationRequiredParam(UniversalBaseModel):
    """
    Param are form input values, primarily utilized when specifying secrets and
    configuration values for actions.

    These are especially important for partner integrations -- but can be
    exposed to tenant admins as well if they want to parameterize their custom
    actions.
    """

    type: typing.Optional[IntegrationRequiredParamTypeEnum] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the parameter.
    """

    required: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The flag for if this parameter is required.
    """

    optional: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The temp flag for if this parameter is required (experimental; for Labs use only).
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    The short label for this parameter.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The lengthier description for this parameter.
    """

    default_value: typing.Optional[str] = pydantic.Field(default=None)
    """
    The default value for this parameter.
    """

    placeholder: typing.Optional[str] = pydantic.Field(default=None)
    """
    Placeholder text for this parameter.
    """

    options: typing.Optional[typing.List[IntegrationRequiredParamOption]] = pydantic.Field(default=None)
    """
    The allowable options for this param.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EventStreamCloudEventUserUpdatedObjectIdentitiesItemDatabaseProfileData(UniversalBaseModel):
    """
    Profile data for the user.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address of this user.
    """

    email_verified: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this email address is verified (true) or unverified (false).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this user.
    """

    username: typing.Optional[str] = pydantic.Field(default=None)
    """
    Username of this user.
    """

    given_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Given name/first name/forename of this user.
    """

    family_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Family name/last name/surname of this user.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Phone number of this user.
    """

    phone_verified: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this phone number has been verified (true) or not (false).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .default_token_quota import DefaultTokenQuota
from .session_cookie_schema import SessionCookieSchema
from .supported_locales import SupportedLocales
from .tenant_oidc_logout_settings import TenantOidcLogoutSettings
from .tenant_settings_country_codes_response import TenantSettingsCountryCodesResponse
from .tenant_settings_device_flow import TenantSettingsDeviceFlow
from .tenant_settings_dynamic_client_registration_security_mode import (
    TenantSettingsDynamicClientRegistrationSecurityMode,
)
from .tenant_settings_error_page import TenantSettingsErrorPage
from .tenant_settings_flags import TenantSettingsFlags
from .tenant_settings_guardian_page import TenantSettingsGuardianPage
from .tenant_settings_mtls import TenantSettingsMtls
from .tenant_settings_nullable_security_headers import TenantSettingsNullableSecurityHeaders
from .tenant_settings_password_page import TenantSettingsPasswordPage
from .tenant_settings_resource_parameter_profile import TenantSettingsResourceParameterProfile
from .tenant_settings_sessions import TenantSettingsSessions


class GetTenantSettingsResponseContent(UniversalBaseModel):
    change_password: typing.Optional[TenantSettingsPasswordPage] = None
    guardian_mfa_page: typing.Optional[TenantSettingsGuardianPage] = None
    default_audience: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default audience for API authorization.
    """

    default_directory: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of connection used for password grants at the `/token`endpoint. The following connection types are supported: LDAP, AD, Database Connections, Passwordless, Windows Azure Active Directory, ADFS.
    """

    error_page: typing.Optional[TenantSettingsErrorPage] = None
    device_flow: typing.Optional[TenantSettingsDeviceFlow] = None
    default_token_quota: typing.Optional[DefaultTokenQuota] = None
    flags: typing.Optional[TenantSettingsFlags] = None
    friendly_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Friendly name for this tenant.
    """

    picture_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of logo to be shown for this tenant (recommended size: 150x150)
    """

    support_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    End-user support email address.
    """

    support_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    End-user support URL.
    """

    allowed_logout_urls: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    URLs that are valid to redirect to after logout from Auth0.
    """

    session_lifetime: typing.Optional[float] = pydantic.Field(default=None)
    """
    Number of hours a session will stay valid.
    """

    idle_session_lifetime: typing.Optional[float] = pydantic.Field(default=None)
    """
    Number of hours for which a session can be inactive before the user must log in again.
    """

    ephemeral_session_lifetime: typing.Optional[float] = pydantic.Field(default=None)
    """
    Number of hours an ephemeral (non-persistent) session will stay valid.
    """

    idle_ephemeral_session_lifetime: typing.Optional[float] = pydantic.Field(default=None)
    """
    Number of hours for which an ephemeral (non-persistent) session can be inactive before the user must log in again.
    """

    sandbox_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Selected sandbox version for the extensibility environment.
    """

    legacy_sandbox_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Selected sandbox version for rules and hooks extensibility.
    """

    sandbox_versions_available: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Available sandbox versions for the extensibility environment.
    """

    default_redirection_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    The default absolute redirection uri, must be https
    """

    enabled_locales: typing.Optional[typing.List[SupportedLocales]] = pydantic.Field(default=None)
    """
    Supported locales for the user interface.
    """

    security_headers: typing.Optional[TenantSettingsNullableSecurityHeaders] = None
    session_cookie: typing.Optional[SessionCookieSchema] = None
    sessions: typing.Optional[TenantSettingsSessions] = None
    oidc_logout: typing.Optional[TenantOidcLogoutSettings] = None
    allow_organization_name_in_authentication_api: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to accept an organization name instead of an ID on auth endpoints
    """

    customize_mfa_in_postlogin_action: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to enable flexible factors for MFA in the PostLogin action
    """

    acr_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Supported ACR values
    """

    mtls: typing.Optional[TenantSettingsMtls] = None
    pushed_authorization_requests_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables the use of Pushed Authorization Requests
    """

    authorization_response_iss_parameter_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Supports iss parameter in authorization responses
    """

    skip_non_verifiable_callback_uri_confirmation_prompt: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Controls whether a confirmation prompt is shown during login flows when the redirect URI uses non-verifiable callback URIs (for example, a custom URI schema such as `myapp://`, or `localhost`).
    If set to true, a confirmation prompt will not be shown. We recommend that this is set to false for improved protection from malicious apps.
    See https://auth0.com/docs/secure/security-guidance/measures-against-app-impersonation for more information.
    """

    resource_parameter_profile: typing.Optional[TenantSettingsResourceParameterProfile] = None
    client_id_metadata_document_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the authorization server supports retrieving client metadata from a client_id URL.
    """

    phone_consolidated_experience: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether Phone Consolidated Experience is enabled for this tenant.
    """

    enable_ai_guide: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether Auth0 Guide (AI-powered assistance) is enabled for this tenant.
    """

    include_session_metadata_in_tenant_logs: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether session metadata is included in specific tenant logs (slo, oidc_backchannel_logout_failed, oidc_backchannel_logout_succeeded).
    """

    dynamic_client_registration_security_mode: typing.Optional[TenantSettingsDynamicClientRegistrationSecurityMode] = (
        None
    )
    country_codes: typing.Optional[TenantSettingsCountryCodesResponse] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_action_response_content import EventStreamActionResponseContent
from .event_stream_event_bridge_response_content import EventStreamEventBridgeResponseContent
from .event_stream_webhook_response_content import EventStreamWebhookResponseContent

GetEventStreamResponseContent = typing.Union[
    EventStreamWebhookResponseContent, EventStreamEventBridgeResponseContent, EventStreamActionResponseContent
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .form_widget_auth_0_verifiable_credentials import FormWidgetAuth0VerifiableCredentials
from .form_widget_g_maps_address import FormWidgetGMapsAddress
from .form_widget_recaptcha import FormWidgetRecaptcha

FormWidget = typing.Union[FormWidgetAuth0VerifiableCredentials, FormWidgetGMapsAddress, FormWidgetRecaptcha]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.list_user_grants_offset_paginated_response_content import ListUserGrantsOffsetPaginatedResponseContent
from ..types.user_grant import UserGrant
from .raw_client import AsyncRawUserGrantsClient, RawUserGrantsClient


class UserGrantsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUserGrantsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUserGrantsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUserGrantsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        per_page: typing.Optional[int] = 50,
        page: typing.Optional[int] = 0,
        include_totals: typing.Optional[bool] = True,
        user_id: typing.Optional[str] = None,
        client_id: typing.Optional[str] = None,
        audience: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserGrant, ListUserGrantsOffsetPaginatedResponseContent]:
        """
        Retrieve the [grants](https://auth0.com/docs/api-auth/which-oauth-flow-to-use) associated with your account.

        Parameters
        ----------
        per_page : typing.Optional[int]
            Number of results per page.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        user_id : typing.Optional[str]
            user_id of the grants to retrieve.

        client_id : typing.Optional[str]
            client_id of the grants to retrieve.

        audience : typing.Optional[str]
            audience of the grants to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserGrant, ListUserGrantsOffsetPaginatedResponseContent]
            Grants successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.user_grants.list(
            per_page=1,
            page=1,
            include_totals=True,
            user_id="user_id",
            client_id="client_id",
            audience="audience",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            per_page=per_page,
            page=page,
            include_totals=include_totals,
            user_id=user_id,
            client_id=client_id,
            audience=audience,
            request_options=request_options,
        )

    def delete_by_user_id(self, *, user_id: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a grant associated with your account.

        Parameters
        ----------
        user_id : str
            user_id of the grant to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.user_grants.delete_by_user_id(
            user_id="user_id",
        )
        """
        _response = self._raw_client.delete_by_user_id(user_id=user_id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a grant associated with your account.

        Parameters
        ----------
        id : str
            ID of the grant to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.user_grants.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncUserGrantsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUserGrantsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUserGrantsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUserGrantsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        per_page: typing.Optional[int] = 50,
        page: typing.Optional[int] = 0,
        include_totals: typing.Optional[bool] = True,
        user_id: typing.Optional[str] = None,
        client_id: typing.Optional[str] = None,
        audience: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserGrant, ListUserGrantsOffsetPaginatedResponseContent]:
        """
        Retrieve the [grants](https://auth0.com/docs/api-auth/which-oauth-flow-to-use) associated with your account.

        Parameters
        ----------
        per_page : typing.Optional[int]
            Number of results per page.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        user_id : typing.Optional[str]
            user_id of the grants to retrieve.

        client_id : typing.Optional[str]
            client_id of the grants to retrieve.

        audience : typing.Optional[str]
            audience of the grants to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserGrant, ListUserGrantsOffsetPaginatedResponseContent]
            Grants successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.user_grants.list(
                per_page=1,
                page=1,
                include_totals=True,
                user_id="user_id",
                client_id="client_id",
                audience="audience",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            per_page=per_page,
            page=page,
            include_totals=include_totals,
            user_id=user_id,
            client_id=client_id,
            audience=audience,
            request_options=request_options,
        )

    async def delete_by_user_id(self, *, user_id: str, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a grant associated with your account.

        Parameters
        ----------
        user_id : str
            user_id of the grant to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_grants.delete_by_user_id(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_by_user_id(user_id=user_id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a grant associated with your account.

        Parameters
        ----------
        id : str
            ID of the grant to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_grants.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .self_service_profile_user_attribute import SelfServiceProfileUserAttribute

SelfServiceProfileUserAttributes = typing.Optional[typing.List[SelfServiceProfileUserAttribute]]
"""
List of attributes to be mapped that will be shown to the user during the Self-Service Enterprise Configuration flow.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .email_provider_credentials_schema_access_key_id import EmailProviderCredentialsSchemaAccessKeyId
from .email_provider_credentials_schema_api_key import EmailProviderCredentialsSchemaApiKey
from .email_provider_credentials_schema_client_id import EmailProviderCredentialsSchemaClientId
from .email_provider_credentials_schema_connection_string import EmailProviderCredentialsSchemaConnectionString
from .email_provider_credentials_schema_smtp_host import EmailProviderCredentialsSchemaSmtpHost
from .email_provider_credentials_schema_three import EmailProviderCredentialsSchemaThree
from .email_provider_credentials_schema_zero import EmailProviderCredentialsSchemaZero
from .extensibility_email_provider_credentials import ExtensibilityEmailProviderCredentials

EmailProviderCredentialsSchema = typing.Union[
    EmailProviderCredentialsSchemaZero,
    EmailProviderCredentialsSchemaAccessKeyId,
    EmailProviderCredentialsSchemaSmtpHost,
    EmailProviderCredentialsSchemaThree,
    EmailProviderCredentialsSchemaApiKey,
    EmailProviderCredentialsSchemaConnectionString,
    EmailProviderCredentialsSchemaClientId,
    ExtensibilityEmailProviderCredentials,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.create_user_authentication_method_response_content import CreateUserAuthenticationMethodResponseContent
from ...types.created_user_authentication_method_type_enum import CreatedUserAuthenticationMethodTypeEnum
from ...types.credential_device_type_enum import CredentialDeviceTypeEnum
from ...types.get_user_authentication_method_response_content import GetUserAuthenticationMethodResponseContent
from ...types.list_user_authentication_methods_offset_paginated_response_content import (
    ListUserAuthenticationMethodsOffsetPaginatedResponseContent,
)
from ...types.preferred_authentication_method_enum import PreferredAuthenticationMethodEnum
from ...types.set_user_authentication_method_response_content import SetUserAuthenticationMethodResponseContent
from ...types.set_user_authentication_methods_request_content import SetUserAuthenticationMethodsRequestContent
from ...types.update_user_authentication_method_response_content import UpdateUserAuthenticationMethodResponseContent
from ...types.user_authentication_method import UserAuthenticationMethod
from .raw_client import AsyncRawAuthenticationMethodsClient, RawAuthenticationMethodsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AuthenticationMethodsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthenticationMethodsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuthenticationMethodsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthenticationMethodsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserAuthenticationMethod, ListUserAuthenticationMethodsOffsetPaginatedResponseContent]:
        """
        Retrieve detailed list of authentication methods associated with a specified user.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0. Default is 0.

        per_page : typing.Optional[int]
            Number of results per page. Default is 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserAuthenticationMethod, ListUserAuthenticationMethodsOffsetPaginatedResponseContent]
            The authentication methods for the user were retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.authentication_methods.list(
            id="id",
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id, page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def create(
        self,
        id: str,
        *,
        type: CreatedUserAuthenticationMethodTypeEnum,
        name: typing.Optional[str] = OMIT,
        totp_secret: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        preferred_authentication_method: typing.Optional[PreferredAuthenticationMethodEnum] = OMIT,
        key_id: typing.Optional[str] = OMIT,
        public_key: typing.Optional[str] = OMIT,
        aaguid: typing.Optional[str] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        credential_device_type: typing.Optional[CredentialDeviceTypeEnum] = OMIT,
        credential_backed_up: typing.Optional[bool] = OMIT,
        identity_user_id: typing.Optional[str] = OMIT,
        user_agent: typing.Optional[str] = OMIT,
        user_handle: typing.Optional[str] = OMIT,
        transports: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateUserAuthenticationMethodResponseContent:
        """
        Create an authentication method. Authentication methods created via this endpoint will be auto confirmed and should already have verification completed.

        Parameters
        ----------
        id : str
            The ID of the user to whom the new authentication method will be assigned.

        type : CreatedUserAuthenticationMethodTypeEnum

        name : typing.Optional[str]
            A human-readable label to identify the authentication method.

        totp_secret : typing.Optional[str]
            Base32 encoded secret for TOTP generation.

        phone_number : typing.Optional[str]
            Applies to phone authentication methods only. The destination phone number used to send verification codes via text and voice.

        email : typing.Optional[str]
            Applies to email authentication methods only. The email address used to send verification messages.

        preferred_authentication_method : typing.Optional[PreferredAuthenticationMethodEnum]

        key_id : typing.Optional[str]
            Applies to webauthn/passkey authentication methods only. The id of the credential.

        public_key : typing.Optional[str]
            Applies to webauthn/passkey authentication methods only. The public key, which is encoded as base64.

        aaguid : typing.Optional[str]
            Applies to passkeys only. Authenticator Attestation Globally Unique Identifier

        relying_party_identifier : typing.Optional[str]
            Applies to webauthn authentication methods only. The relying party identifier.

        credential_device_type : typing.Optional[CredentialDeviceTypeEnum]

        credential_backed_up : typing.Optional[bool]
            Applies to passkeys only. Whether the credential was backed up.

        identity_user_id : typing.Optional[str]
            Applies to passkeys only. The ID of the user identity linked with the authentication method.

        user_agent : typing.Optional[str]
            Applies to passkeys only. The user-agent of the browser used to create the passkey.

        user_handle : typing.Optional[str]
            Applies to passkeys only. The user handle of the user identity.

        transports : typing.Optional[typing.Sequence[str]]
            Applies to passkeys only. The transports used by clients to communicate with the authenticator.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateUserAuthenticationMethodResponseContent
            Authentication method created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authentication_methods.create(
            id="id",
            type="phone",
        )
        """
        _response = self._raw_client.create(
            id,
            type=type,
            name=name,
            totp_secret=totp_secret,
            phone_number=phone_number,
            email=email,
            preferred_authentication_method=preferred_authentication_method,
            key_id=key_id,
            public_key=public_key,
            aaguid=aaguid,
            relying_party_identifier=relying_party_identifier,
            credential_device_type=credential_device_type,
            credential_backed_up=credential_backed_up,
            identity_user_id=identity_user_id,
            user_agent=user_agent,
            user_handle=user_handle,
            transports=transports,
            request_options=request_options,
        )
        return _response.data

    def set(
        self,
        id: str,
        *,
        request: SetUserAuthenticationMethodsRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SetUserAuthenticationMethodResponseContent]:
        """
        Replace the specified user <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors"> authentication methods</a> with supplied values.

            <b>Note</b>: Authentication methods supplied through this action do not iterate on existing methods. Instead, any methods passed will overwrite the user&#8217s existing settings.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        request : SetUserAuthenticationMethodsRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SetUserAuthenticationMethodResponseContent]
            All authentication methods successfully updated.

        Examples
        --------
        from auth0 import Auth0, SetUserAuthenticationMethods

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authentication_methods.set(
            id="id",
            request=[
                SetUserAuthenticationMethods(
                    type="phone",
                )
            ],
        )
        """
        _response = self._raw_client.set(id, request=request, request_options=request_options)
        return _response.data

    def delete_all(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all authentication methods (i.e., enrolled MFA factors) from the specified user account. This action cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authentication_methods.delete_all(
            id="id",
        )
        """
        _response = self._raw_client.delete_all(id, request_options=request_options)
        return _response.data

    def get(
        self, id: str, authentication_method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetUserAuthenticationMethodResponseContent:
        """
        Parameters
        ----------
        id : str
            The ID of the user in question.

        authentication_method_id : str
            The ID of the authentication methods in question.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserAuthenticationMethodResponseContent
            Authentication method retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authentication_methods.get(
            id="id",
            authentication_method_id="authentication_method_id",
        )
        """
        _response = self._raw_client.get(id, authentication_method_id, request_options=request_options)
        return _response.data

    def delete(
        self, id: str, authentication_method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove the authentication method with the given ID from the specified user. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/manage-mfa-auth0-apis/manage-authentication-methods-with-management-api">Manage Authentication Methods with Management API</a>.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        authentication_method_id : str
            The ID of the authentication method to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authentication_methods.delete(
            id="id",
            authentication_method_id="authentication_method_id",
        )
        """
        _response = self._raw_client.delete(id, authentication_method_id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        authentication_method_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        preferred_authentication_method: typing.Optional[PreferredAuthenticationMethodEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateUserAuthenticationMethodResponseContent:
        """
        Modify the authentication method with the given ID from the specified user. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/manage-mfa-auth0-apis/manage-authentication-methods-with-management-api">Manage Authentication Methods with Management API</a>.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        authentication_method_id : str
            The ID of the authentication method to update.

        name : typing.Optional[str]
            A human-readable label to identify the authentication method.

        preferred_authentication_method : typing.Optional[PreferredAuthenticationMethodEnum]
            Preferred phone authentication method

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUserAuthenticationMethodResponseContent
            Authentication method updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authentication_methods.update(
            id="id",
            authentication_method_id="authentication_method_id",
        )
        """
        _response = self._raw_client.update(
            id,
            authentication_method_id,
            name=name,
            preferred_authentication_method=preferred_authentication_method,
            request_options=request_options,
        )
        return _response.data


class AsyncAuthenticationMethodsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthenticationMethodsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuthenticationMethodsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthenticationMethodsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserAuthenticationMethod, ListUserAuthenticationMethodsOffsetPaginatedResponseContent]:
        """
        Retrieve detailed list of authentication methods associated with a specified user.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0. Default is 0.

        per_page : typing.Optional[int]
            Number of results per page. Default is 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserAuthenticationMethod, ListUserAuthenticationMethodsOffsetPaginatedResponseContent]
            The authentication methods for the user were retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.authentication_methods.list(
                id="id",
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id, page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def create(
        self,
        id: str,
        *,
        type: CreatedUserAuthenticationMethodTypeEnum,
        name: typing.Optional[str] = OMIT,
        totp_secret: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        email: typing.Optional[str] = OMIT,
        preferred_authentication_method: typing.Optional[PreferredAuthenticationMethodEnum] = OMIT,
        key_id: typing.Optional[str] = OMIT,
        public_key: typing.Optional[str] = OMIT,
        aaguid: typing.Optional[str] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        credential_device_type: typing.Optional[CredentialDeviceTypeEnum] = OMIT,
        credential_backed_up: typing.Optional[bool] = OMIT,
        identity_user_id: typing.Optional[str] = OMIT,
        user_agent: typing.Optional[str] = OMIT,
        user_handle: typing.Optional[str] = OMIT,
        transports: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateUserAuthenticationMethodResponseContent:
        """
        Create an authentication method. Authentication methods created via this endpoint will be auto confirmed and should already have verification completed.

        Parameters
        ----------
        id : str
            The ID of the user to whom the new authentication method will be assigned.

        type : CreatedUserAuthenticationMethodTypeEnum

        name : typing.Optional[str]
            A human-readable label to identify the authentication method.

        totp_secret : typing.Optional[str]
            Base32 encoded secret for TOTP generation.

        phone_number : typing.Optional[str]
            Applies to phone authentication methods only. The destination phone number used to send verification codes via text and voice.

        email : typing.Optional[str]
            Applies to email authentication methods only. The email address used to send verification messages.

        preferred_authentication_method : typing.Optional[PreferredAuthenticationMethodEnum]

        key_id : typing.Optional[str]
            Applies to webauthn/passkey authentication methods only. The id of the credential.

        public_key : typing.Optional[str]
            Applies to webauthn/passkey authentication methods only. The public key, which is encoded as base64.

        aaguid : typing.Optional[str]
            Applies to passkeys only. Authenticator Attestation Globally Unique Identifier

        relying_party_identifier : typing.Optional[str]
            Applies to webauthn authentication methods only. The relying party identifier.

        credential_device_type : typing.Optional[CredentialDeviceTypeEnum]

        credential_backed_up : typing.Optional[bool]
            Applies to passkeys only. Whether the credential was backed up.

        identity_user_id : typing.Optional[str]
            Applies to passkeys only. The ID of the user identity linked with the authentication method.

        user_agent : typing.Optional[str]
            Applies to passkeys only. The user-agent of the browser used to create the passkey.

        user_handle : typing.Optional[str]
            Applies to passkeys only. The user handle of the user identity.

        transports : typing.Optional[typing.Sequence[str]]
            Applies to passkeys only. The transports used by clients to communicate with the authenticator.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateUserAuthenticationMethodResponseContent
            Authentication method created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authentication_methods.create(
                id="id",
                type="phone",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            id,
            type=type,
            name=name,
            totp_secret=totp_secret,
            phone_number=phone_number,
            email=email,
            preferred_authentication_method=preferred_authentication_method,
            key_id=key_id,
            public_key=public_key,
            aaguid=aaguid,
            relying_party_identifier=relying_party_identifier,
            credential_device_type=credential_device_type,
            credential_backed_up=credential_backed_up,
            identity_user_id=identity_user_id,
            user_agent=user_agent,
            user_handle=user_handle,
            transports=transports,
            request_options=request_options,
        )
        return _response.data

    async def set(
        self,
        id: str,
        *,
        request: SetUserAuthenticationMethodsRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SetUserAuthenticationMethodResponseContent]:
        """
        Replace the specified user <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors"> authentication methods</a> with supplied values.

            <b>Note</b>: Authentication methods supplied through this action do not iterate on existing methods. Instead, any methods passed will overwrite the user&#8217s existing settings.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        request : SetUserAuthenticationMethodsRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SetUserAuthenticationMethodResponseContent]
            All authentication methods successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, SetUserAuthenticationMethods

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authentication_methods.set(
                id="id",
                request=[
                    SetUserAuthenticationMethods(
                        type="phone",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(id, request=request, request_options=request_options)
        return _response.data

    async def delete_all(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all authentication methods (i.e., enrolled MFA factors) from the specified user account. This action cannot be undone.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authentication_methods.delete_all(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_all(id, request_options=request_options)
        return _response.data

    async def get(
        self, id: str, authentication_method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetUserAuthenticationMethodResponseContent:
        """
        Parameters
        ----------
        id : str
            The ID of the user in question.

        authentication_method_id : str
            The ID of the authentication methods in question.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserAuthenticationMethodResponseContent
            Authentication method retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authentication_methods.get(
                id="id",
                authentication_method_id="authentication_method_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, authentication_method_id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, authentication_method_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove the authentication method with the given ID from the specified user. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/manage-mfa-auth0-apis/manage-authentication-methods-with-management-api">Manage Authentication Methods with Management API</a>.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        authentication_method_id : str
            The ID of the authentication method to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authentication_methods.delete(
                id="id",
                authentication_method_id="authentication_method_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, authentication_method_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        authentication_method_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        preferred_authentication_method: typing.Optional[PreferredAuthenticationMethodEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateUserAuthenticationMethodResponseContent:
        """
        Modify the authentication method with the given ID from the specified user. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/manage-mfa-auth0-apis/manage-authentication-methods-with-management-api">Manage Authentication Methods with Management API</a>.

        Parameters
        ----------
        id : str
            The ID of the user in question.

        authentication_method_id : str
            The ID of the authentication method to update.

        name : typing.Optional[str]
            A human-readable label to identify the authentication method.

        preferred_authentication_method : typing.Optional[PreferredAuthenticationMethodEnum]
            Preferred phone authentication method

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUserAuthenticationMethodResponseContent
            Authentication method updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authentication_methods.update(
                id="id",
                authentication_method_id="authentication_method_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            authentication_method_id,
            name=name,
            preferred_authentication_method=preferred_authentication_method,
            request_options=request_options,
        )
        return _response.data



import base64
import json
import time
from unittest.mock import MagicMock, patch

import httpx
import pytest

from auth0.management import (
    AsyncManagementClient,
    AsyncTokenProvider,
    CustomDomainHeader,
    ManagementClient,
    TokenProvider,
)
from auth0.management.management_client import (
    CUSTOM_DOMAIN_HEADER as _CUSTOM_DOMAIN_HEADER,
)
from auth0.management.management_client import (
    _enforce_custom_domain_whitelist,
    _is_path_whitelisted,
)


class TestManagementClientInit:
    """Tests for ManagementClient initialization."""

    def test_init_with_token_string(self):
        """Should initialize with a static token string."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-test-token",
        )
        assert client._api is not None

    def test_init_with_token_callable(self):
        """Should initialize with a token callable."""
        token_fn = lambda: "dynamic-token"
        client = ManagementClient(
            domain="test.auth0.com",
            token=token_fn,
        )
        assert client._api is not None

    def test_init_with_client_credentials(self):
        """Should initialize with client_id and client_secret."""
        client = ManagementClient(
            domain="test.auth0.com",
            client_id="my-client-id",
            client_secret="my-client-secret",
        )
        assert client._api is not None

    def test_init_with_custom_audience(self):
        """Should initialize with a custom audience."""
        client = ManagementClient(
            domain="test.auth0.com",
            client_id="my-client-id",
            client_secret="my-client-secret",
            audience="https://custom-api.example.com/",
        )
        assert client._api is not None

    def test_init_requires_auth(self):
        """Should raise ValueError when no auth is provided."""
        with pytest.raises(ValueError) as exc_info:
            ManagementClient(domain="test.auth0.com")
        assert "token" in str(exc_info.value)
        assert "client_id" in str(exc_info.value)

    def test_init_requires_both_credentials(self):
        """Should raise ValueError when only client_id is provided."""
        with pytest.raises(ValueError):
            ManagementClient(
                domain="test.auth0.com",
                client_id="my-client-id",
            )

    def test_init_with_custom_timeout(self):
        """Should initialize with custom timeout."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
            timeout=30.0,
        )
        assert client._api is not None

    def test_init_with_custom_headers(self):
        """Should initialize with custom headers."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
            headers={"X-Custom-Header": "custom-value"},
        )
        assert client._api is not None

    def test_init_with_custom_client_info(self):
        """Should encode client_info as Auth0-Client header."""
        custom_info = {
            "name": "auth0-ai-langchain",
            "version": "1.0.0",
            "env": {"python": "3.11.0"},
        }
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
            client_info=custom_info,
        )
        # Verify the header was set on the underlying client wrapper
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is not None
        encoded_header = custom_headers.get("Auth0-Client")
        assert encoded_header is not None
        decoded = json.loads(base64.b64decode(encoded_header).decode("utf-8"))
        assert decoded == custom_info

    def test_init_with_client_info_and_custom_headers(self):
        """Should merge client_info with custom headers."""
        custom_info = {"name": "my-sdk", "version": "2.0.0"}
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
            headers={"X-Custom": "value"},
            client_info=custom_info,
        )
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is not None
        assert custom_headers.get("X-Custom") == "value"
        assert "Auth0-Client" in custom_headers

    def test_init_without_client_info_uses_default_telemetry(self):
        """Should use default auth0-python telemetry when client_info is not provided."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
        )
        # get_headers() includes the default Auth0-Client telemetry
        headers = client._api._client_wrapper.get_headers()
        encoded = headers.get("Auth0-Client")
        assert encoded is not None
        decoded = json.loads(base64.b64decode(encoded).decode("utf-8"))
        assert decoded["name"] == "auth0-python"


class TestManagementClientProperties:
    """Tests for ManagementClient sub-client properties."""

    @pytest.fixture
    def client(self):
        return ManagementClient(
            domain="test.auth0.com",
            token="test-token",
        )

    def test_has_users_property(self, client):
        """Should have users property."""
        assert hasattr(client, "users")

    def test_has_organizations_property(self, client):
        """Should have organizations property."""
        assert hasattr(client, "organizations")

    def test_has_actions_property(self, client):
        """Should have actions property."""
        assert hasattr(client, "actions")

    def test_has_all_expected_properties(self, client):
        """Should have all expected sub-client properties."""
        expected_properties = [
            "actions",
            "anomaly",
            "attack_protection",
            "branding",
            "client_grants",
            "clients",
            "connections",
            "custom_domains",
            "device_credentials",
            "email_templates",
            "emails",
            "event_streams",
            "flows",
            "forms",
            "guardian",
            "hooks",
            "jobs",
            "keys",
            "log_streams",
            "logs",
            "network_acls",
            "organizations",
            "prompts",
            "refresh_tokens",
            "resource_servers",
            "roles",
            "rules",
            "rules_configs",
            "self_service_profiles",
            "sessions",
            "stats",
            "supplemental_signals",
            "tenants",
            "tickets",
            "token_exchange_profiles",
            "user_blocks",
            "user_grants",
            "users",
            "verifiable_credentials",
        ]
        for prop in expected_properties:
            assert hasattr(client, prop), f"Missing property: {prop}"


class TestAsyncManagementClientInit:
    """Tests for AsyncManagementClient initialization."""

    def test_init_with_token_string(self):
        """Should initialize with a static token string."""
        client = AsyncManagementClient(
            domain="test.auth0.com",
            token="my-test-token",
        )
        assert client._api is not None

    def test_init_with_client_credentials(self):
        """Should initialize with client_id and client_secret."""
        client = AsyncManagementClient(
            domain="test.auth0.com",
            client_id="my-client-id",
            client_secret="my-client-secret",
        )
        assert client._api is not None

    def test_init_requires_auth(self):
        """Should raise ValueError when no auth is provided."""
        with pytest.raises(ValueError):
            AsyncManagementClient(domain="test.auth0.com")

    def test_init_with_custom_client_info(self):
        """Should encode client_info as Auth0-Client header."""
        custom_info = {
            "name": "auth0-ai-langchain",
            "version": "1.0.0",
            "env": {"python": "3.11.0"},
        }
        client = AsyncManagementClient(
            domain="test.auth0.com",
            token="my-token",
            client_info=custom_info,
        )
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is not None
        encoded_header = custom_headers.get("Auth0-Client")
        assert encoded_header is not None
        decoded = json.loads(base64.b64decode(encoded_header).decode("utf-8"))
        assert decoded == custom_info


class TestTokenProvider:
    """Tests for TokenProvider."""

    def test_init_with_defaults(self):
        """Should initialize with default audience."""
        provider = TokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
        )
        assert provider._audience == "https://test.auth0.com/api/v2/"

    def test_init_with_custom_audience(self):
        """Should initialize with custom audience."""
        provider = TokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
            audience="https://custom.api.com/",
        )
        assert provider._audience == "https://custom.api.com/"

    @patch("auth0.management.token_provider.httpx.Client")
    def test_get_token_fetches_on_first_call(self, mock_client_class):
        """Should fetch token on first call."""
        mock_response = MagicMock()
        mock_response.json.return_value = {
            "access_token": "new-token",
            "expires_in": 86400,
        }
        mock_client = MagicMock()
        mock_client.__enter__ = MagicMock(return_value=mock_client)
        mock_client.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = mock_response
        mock_client_class.return_value = mock_client

        provider = TokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
        )
        token = provider.get_token()

        assert token == "new-token"
        mock_client.post.assert_called_once()

    @patch("auth0.management.token_provider.httpx.Client")
    def test_get_token_returns_cached_token(self, mock_client_class):
        """Should return cached token on subsequent calls."""
        mock_response = MagicMock()
        mock_response.json.return_value = {
            "access_token": "cached-token",
            "expires_in": 86400,
        }
        mock_client = MagicMock()
        mock_client.__enter__ = MagicMock(return_value=mock_client)
        mock_client.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = mock_response
        mock_client_class.return_value = mock_client

        provider = TokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
        )

        # First call fetches
        token1 = provider.get_token()
        # Second call returns cached
        token2 = provider.get_token()

        assert token1 == token2 == "cached-token"
        # Should only fetch once
        assert mock_client.post.call_count == 1

    @patch("auth0.management.token_provider.httpx.Client")
    def test_get_token_refreshes_expired_token(self, mock_client_class):
        """Should refresh token when expired (with 10s leeway)."""
        mock_response = MagicMock()
        mock_response.json.return_value = {
            "access_token": "refreshed-token",
            "expires_in": 86400,
        }
        mock_client = MagicMock()
        mock_client.__enter__ = MagicMock(return_value=mock_client)
        mock_client.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = mock_response
        mock_client_class.return_value = mock_client

        provider = TokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
        )

        # First call
        provider.get_token()

        # Simulate token being close to expiration (within leeway)
        provider._expires_at = time.time() + 5  # 5 seconds left, less than 10s leeway

        # This should trigger a refresh
        token = provider.get_token()

        assert token == "refreshed-token"
        assert mock_client.post.call_count == 2

    @patch("auth0.management.token_provider.httpx.Client")
    def test_token_request_payload(self, mock_client_class):
        """Should send correct payload to token endpoint."""
        mock_response = MagicMock()
        mock_response.json.return_value = {
            "access_token": "token",
            "expires_in": 86400,
        }
        mock_client = MagicMock()
        mock_client.__enter__ = MagicMock(return_value=mock_client)
        mock_client.__exit__ = MagicMock(return_value=False)
        mock_client.post.return_value = mock_response
        mock_client_class.return_value = mock_client

        provider = TokenProvider(
            domain="test.auth0.com",
            client_id="my-client",
            client_secret="my-secret",
            audience="https://api.example.com/",
        )
        provider.get_token()

        mock_client.post.assert_called_once_with(
            "https://test.auth0.com/oauth/token",
            data={
                "grant_type": "client_credentials",
                "client_id": "my-client",
                "client_secret": "my-secret",
                "audience": "https://api.example.com/",
            },
        )


class TestAsyncTokenProvider:
    """Tests for AsyncTokenProvider."""

    def test_init_with_defaults(self):
        """Should initialize with default audience."""
        provider = AsyncTokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
        )
        assert provider._audience == "https://test.auth0.com/api/v2/"

    def test_init_with_custom_audience(self):
        """Should initialize with custom audience."""
        provider = AsyncTokenProvider(
            domain="test.auth0.com",
            client_id="client-id",
            client_secret="client-secret",
            audience="https://custom.api.com/",
        )
        assert provider._audience == "https://custom.api.com/"


class TestCustomDomainHeader:
    """Tests for Auth0-Custom-Domain header support."""

    def test_init_with_custom_domain(self):
        """Should set Auth0-Custom-Domain header when custom_domain is provided."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
            custom_domain="login.mycompany.com",
        )
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is not None
        assert custom_headers[_CUSTOM_DOMAIN_HEADER] == "login.mycompany.com"

    def test_init_custom_domain_with_existing_headers(self):
        """Should merge custom_domain with other custom headers."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
            headers={"X-Custom": "value"},
            custom_domain="login.mycompany.com",
        )
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is not None
        assert custom_headers["X-Custom"] == "value"
        assert custom_headers[_CUSTOM_DOMAIN_HEADER] == "login.mycompany.com"

    def test_init_without_custom_domain(self):
        """Should not set Auth0-Custom-Domain header when custom_domain is not provided."""
        client = ManagementClient(
            domain="test.auth0.com",
            token="my-token",
        )
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is None or _CUSTOM_DOMAIN_HEADER not in custom_headers

    def test_custom_domain_header_helper(self):
        """Should return correct request options dict."""
        result = CustomDomainHeader("login.mycompany.com")
        assert result == {
            "additional_headers": {
                _CUSTOM_DOMAIN_HEADER: "login.mycompany.com",
            }
        }

    def test_async_init_with_custom_domain(self):
        """Should set Auth0-Custom-Domain header on async client."""
        client = AsyncManagementClient(
            domain="test.auth0.com",
            token="my-token",
            custom_domain="login.mycompany.com",
        )
        custom_headers = client._api._client_wrapper.get_custom_headers()
        assert custom_headers is not None
        assert custom_headers[_CUSTOM_DOMAIN_HEADER] == "login.mycompany.com"

    def test_whitelist_strips_header_on_non_whitelisted_path(self):
        """Should strip Auth0-Custom-Domain header on non-whitelisted paths."""
        request = httpx.Request(
            "GET",
            "https://test.auth0.com/api/v2/clients",
            headers={_CUSTOM_DOMAIN_HEADER: "login.mycompany.com"},
        )
        _enforce_custom_domain_whitelist(request)
        assert _CUSTOM_DOMAIN_HEADER not in request.headers

    def test_whitelist_keeps_header_on_whitelisted_path(self):
        """Should keep Auth0-Custom-Domain header on whitelisted paths."""
        request = httpx.Request(
            "POST",
            "https://test.auth0.com/api/v2/users",
            headers={_CUSTOM_DOMAIN_HEADER: "login.mycompany.com"},
        )
        _enforce_custom_domain_whitelist(request)
        assert request.headers[_CUSTOM_DOMAIN_HEADER] == "login.mycompany.com"

    @pytest.mark.parametrize(
        "path",
        [
            "/api/v2/jobs/verification-email",
            "/api/v2/tickets/email-verification",
            "/api/v2/tickets/password-change",
            "/api/v2/organizations/org_abc123/invitations",
            "/api/v2/users",
            "/api/v2/users/auth0|abc123",
            "/api/v2/guardian/enrollments/ticket",
            "/api/v2/self-service-profiles/ssp_abc123/sso-ticket",
        ],
    )
    def test_whitelisted_paths_match(self, path):
        """Should match all 8 whitelisted path patterns."""
        assert _is_path_whitelisted(path) is True

    @pytest.mark.parametrize(
        "path",
        [
            "/api/v2/clients",
            "/api/v2/connections",
            "/api/v2/roles",
            "/api/v2/users/auth0|abc123/roles",
            "/api/v2/jobs/users-imports",
            "/api/v2/tenants/settings",
        ],
    )
    def test_non_whitelisted_paths_do_not_match(self, path):
        """Should not match non-whitelisted paths."""
        assert _is_path_whitelisted(path) is False


class TestImports:
    """Tests for module imports."""

    def test_import_from_management(self):
        """Should be able to import from auth0.management."""
        from auth0.management import (
            AsyncManagementClient,
            AsyncTokenProvider,
            CustomDomainHeader,
            ManagementClient,
            TokenProvider,
        )

        assert ManagementClient is not None
        assert AsyncManagementClient is not None
        assert TokenProvider is not None
        assert AsyncTokenProvider is not None
        assert CustomDomainHeader is not None



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.pagination import AsyncPager, SyncPager
from ....core.request_options import RequestOptions
from ....types.list_organization_role_members_response_content import ListOrganizationRoleMembersResponseContent
from ....types.role_member import RoleMember
from .raw_client import AsyncRawMembersClient, RawMembersClient


class MembersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMembersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMembersClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        role_id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[RoleMember, ListOrganizationRoleMembersResponseContent]:
        """
        List the organization members assigned a specific role within the context of an organization.

        Parameters
        ----------
        id : str
            ID of the organization.

        role_id : str
            ID of the role to retrieve the assigned members for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50. Values above the maximum permitted size are capped.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RoleMember, ListOrganizationRoleMembersResponseContent]
            Role members successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.roles.members.list(
            id="id",
            role_id="role_id",
            from_="from",
            take=1,
            fields="fields",
            include_fields=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id,
            role_id,
            from_=from_,
            take=take,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )


class AsyncMembersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMembersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMembersClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        role_id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[RoleMember, ListOrganizationRoleMembersResponseContent]:
        """
        List the organization members assigned a specific role within the context of an organization.

        Parameters
        ----------
        id : str
            ID of the organization.

        role_id : str
            ID of the role to retrieve the assigned members for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50. Values above the maximum permitted size are capped.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RoleMember, ListOrganizationRoleMembersResponseContent]
            Role members successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.organizations.roles.members.list(
                id="id",
                role_id="role_id",
                from_="from",
                take=1,
                fields="fields",
                include_fields=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id,
            role_id,
            from_=from_,
            take=take,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventGroupMemberDeletedCloudEventTypeEnum = typing.Union[
    typing.Literal["group.member.deleted"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

FormWidgetTypeAuth0VerifiableCredentialsConst = typing.Union[typing.Literal["AUTH0_VERIFIABLE_CREDENTIALS"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .verifiable_credential_template_response import VerifiableCredentialTemplateResponse


class ListVerifiableCredentialTemplatesPaginatedResponseContent(UniversalBaseModel):
    next: typing.Optional[str] = pydantic.Field(default=None)
    """
    Opaque identifier for use with the <i>from</i> query parameter for the next page of results.<br/>This identifier is valid for 24 hours.
    """

    templates: typing.Optional[typing.List[VerifiableCredentialTemplateResponse]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionDpopSigningAlgValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWS signing algorithms (alg values) supported for DPoP proof JWT signing.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .suspicious_ip_throttling_allowlist_item import SuspiciousIpThrottlingAllowlistItem

SuspiciousIpThrottlingAllowlist = typing.List[SuspiciousIpThrottlingAllowlistItem]
"""
List of trusted IP addresses that will not have attack protection enforced against them.
"""
